//-----------------------------------------------------------------
// SX15-Evo
// Versione : 1.0
// Area SX
// http://www.areasx.com
// Note     : Programma minimale per la comunicazione con il modem GSM
//-----------------------------------------------------------------
//Buffer Seriale
#define CINBUFSIZE  511
#define COUTBUFSIZE 511
#define CBITRATE 9600

#define GSM_ISOFF       (!BitRdPortI(PEDR, 4))
#define GSM_POWERON     BitWrPortI(PEDR, &PEDRShadow,1,7);msDelay(2000);BitWrPortI(PEDR, &PEDRShadow,0,7)
#define GSM_CLEARRESET  BitWrPortI(PFDR,&PFDRShadow,1,5)   //Alza la linea del reset


//-----------------------------------------------------------------
// Define Hardware
//-----------------------------------------------------------------
#define LED1_ON       BitWrPortI(PFDR,&PFDRShadow,1,0)
#define LED1_OFF      BitWrPortI(PFDR,&PFDRShadow,0,0)
#define LED2_ON       BitWrPortI(PFDR,&PFDRShadow,1,1)
#define LED2_OFF      BitWrPortI(PFDR,&PFDRShadow,0,1)
#define LED3_ON       BitWrPortI(PFDR,&PFDRShadow,1,4)
#define LED3_OFF      BitWrPortI(PFDR,&PFDRShadow,0,4)

#define KEY_PRESSED   BitRdPortI(PBDR,0)


//-----------------------------------------------------------------
// Prototipi delle funzioni
//-----------------------------------------------------------------
void msDelay(unsigned int);
void BoardInit(void);

//-----------------------------------------------------------------
// Variabili globali
//-----------------------------------------------------------------
char txCbuff[CINBUFSIZE];
char rxCbuff[CINBUFSIZE];


//--------------------------------------------------------
// Ritardo bloccante
// Riceve      : ams - numero di millisecondi da ritardare
// Restituisce : -
// Note        :
//--------------------------------------------------------
nodebug
void msDelay(unsigned int ams)
{
  auto unsigned long t0;
  for(t0=MS_TIMER; MS_TIMER<ams+t0;);
}

//--------------------------------------------------------
// Inizializza i port del modulo RCM3700
// Riceve      : -
// Restituisce : -
// Note        :
//--------------------------------------------------------
void BoardInit() {

  // Port A
  // Default ingresso

  // Port B
  // Default

  // Port C
  // PC0-PC1 - Seriale D
  BitWrPortI(PCFR, &PCFRShadow, 1,0);

  // PC2-PC3 - Seriale C
  // Modem GSM
  BitWrPortI(PCFR, &PCFRShadow, 1,2);

  // Port D
  // PD4-PD5 - Seriale B

  // Port E
  WrPortI(PEFR, &PEFRShadow, 0x00);
  // PE0 - DCD Modem GSM (ingresso)
  // PE1 - Pulsante esterno (ingresso)
  // PE4 - GSM Power Control (ingresso)
  // PE5 - RS485 Data Direction / RS232 DTR (output)
  BitWrPortI(PEDDR, &PEDDRShadow, 1,5);
  // PE7 - GSM On/Off (output)
  BitWrPortI(PEDDR, &PEDDRShadow, 1,7);

  // Port F
  WrPortI(PFFR, &PFFRShadow, 0x00);
  WrPortI(PFDCR, &PFDCRShadow, 0x00);
  // PF0 - Led LD3 (output)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,0);
  // PF1 - Led LD1 (output)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,1);
  // PF4 - Led LD2 (output)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,4);
  // PF5 - Modem Reset (output)
  BitWrPortI(PFDDR, &PFDDRShadow, 1,5);
  // PF6 - SH71 External I2c (bid)
  // PF7 - SH71 External I2c (bid)

  // Port G
  WrPortI(PGFR, &PGFRShadow, 0x00);
  // PG6-PG7 - Seriale E (RF Transmitter, Caller Id, GPS Receiver)
  BitWrPortI(PGFR, &PGFRShadow, 1,6);
  BitWrPortI(PGDDR, &PGDDRShadow, 0,7);
  BitWrPortI(PGDDR, &PGDDRShadow, 1,6);

  GSM_CLEARRESET;
  msDelay(1000);

}


int SendGsmCommand(char *cmd,int timeout,int mode) {
  int i,len;

  // Pulizia buffer di ricezione
    serCrdFlush();
    serCwrFlush();
   rxCbuff[0]=0;
   txCbuff[0]=0;

  if (mode){
    sprintf(txCbuff,"%s",cmd);
  } else {
      sprintf(txCbuff,"%s%c",cmd,13);
   }
   printf("Invio al modem: %s", txCbuff);
   serCputs(txCbuff);
  msDelay(timeout);
  // Attesa di risposta

  i=serCread(rxCbuff,511,1000);
   printf("caratteri ricevuti %d\n",i);
  if (i>0) {
      rxCbuff[i-1]=0;  // Metto un fine stringa al posto di CR
      printf("Risposta DAL GSM: %s\n",rxCbuff);
      printf ("Send Gsm OK!\n");
      return (i);
  } else {
      printf ("Send Gsm ERROR\n",i);
      return(0);
  }
}



//////////////////////////////////////////////////
//
// MAIN
//
/////////////////////////////////////////////////

void main(){
   int tmp;

   printf("Firmware di test per la scheda SX15-Evo con GSM\n");
  BoardInit();
  serCopen(9600); //Apre la porta seriale C collegata al modem

  // Accensione terminale GSM se non acceso
  if (GSM_ISOFF) {
    #ifdef GSM_LIBDEBUG
      printf("Powering on GSM\n");
    #endif

    // Accensione engine
    GSM_POWERON;
    msDelay(500);
  } else {
    #ifdef GSM_LIBDEBUG
      printf("Modem Power OK\n");
    #endif
  }

  msDelay(2000);

  while(1){

    printf("INVIO COMANDO AT MODEM\n");

    SendGsmCommand("AT",1500,0);

    if (!strstr(rxCbuff,"OK")) {
      printf("Nessuna risposta dal modem\nRipeto il comando di start\n");

      msDelay(5000);
      // Accensione terminale GSM se non acceso
      if (GSM_ISOFF) {
        #ifdef GSM_LIBDEBUG
          printf("Powering on GSM\n");
        #endif


        // Accensione engine
        GSM_POWERON;
        msDelay(500);
      } else {
        #ifdef GSM_LIBDEBUG
          printf("Modem Power OK\n");
        #endif
      }



    } else {
      // Disabilitazione ECHO
      printf("Disabilitazione ECHO\n");
      SendGsmCommand("ATE0",1500,0);

      if (strstr(rxCbuff,"OK")) {
        printf ("Modem pronto!!\n");
      } else {
        printf ("Risposta errata dal modem\n");
      }

    }

    msDelay(3000);

  }

}